@@ -59,13 +59,6 @@ template<typename Integer> class Matrix {
 //              Private routines, used in the public routines
 //---------------------------------------------------------------------------
 
-//---------------------------------------------------------------------------
-//                      Rows and columns exchange
-//---------------------------------------------------------------------------
-
-    void exchange_rows(const size_t& row1, const size_t& row2);      //row1 is exchanged with row2
-    void exchange_columns(const size_t& col1, const size_t& col2); // col1 is exchanged with col2
-
 //---------------------------------------------------------------------------
 //              Row and column reduction
 //---------------------------------------------------------------------------
@@ -123,6 +116,8 @@ template<typename Integer> class Matrix {
     
     bool SmithNormalForm_inner(size_t& rk, Matrix<Integer>& Right);
     
+    vector<Integer> optimal_subdivision_point_inner() const;
+    
 
 //---------------------------------------------------------------------------
 //                      Pivots for rows/columns operations
@@ -152,7 +147,14 @@ public:
 
 size_t row_echelon_inner_bareiss(bool& success, Integer& det);
 
-    vector<vector<Integer>* > submatrix_pointers(const vector<key_t>& key);     
+    vector<vector<Integer>* > submatrix_pointers(const vector<key_t>& key);
+    
+//---------------------------------------------------------------------------
+//                      Rows and columns exchange
+//---------------------------------------------------------------------------
+
+    void exchange_rows(const size_t& row1, const size_t& row2);      //row1 is exchanged with row2
+    void exchange_columns(const size_t& col1, const size_t& col2); // col1 is exchanged with col2
   
 //---------------------------------------------------------------------------
 
@@ -174,7 +176,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     void write_column(size_t col, const vector<Integer>& data); //write a column
     void print(const string& name, const string& suffix) const;         //  writes matrix into name.suffix
     void print_append(const string& name,const string& suffix) const;  // the same, but appends matrix
-    void print(std::ostream& out) const;          // writes matrix to the stream
+    void print(std::ostream& out, bool with_format=false) const;          // writes matrix to the stream
     void pretty_print(std::ostream& out, bool with_row_nr=false) const;  // writes matrix in a nice format to the stream                   // read a row
     size_t nr_of_rows() const;                       // returns nr
     size_t nr_of_columns() const;                   // returns nc
@@ -223,7 +225,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     void append_column(const vector<Integer>& v); // append the column v to this
     void remove_row(const vector<Integer>& row); // removes all appearances of this row, not very efficient!
     void remove_row(const size_t index);
-    void remove_duplicate_and_zero_rows();
+    vector<size_t> remove_duplicate_and_zero_rows();
     void remove_duplicate(const Matrix& M);
 
 
@@ -253,8 +255,6 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     Matrix add(const Matrix& A) const;                       // returns this+A
     Matrix multiplication(const Matrix& A) const;          // returns this*A
     Matrix multiplication(const Matrix& A, long m) const;// returns this*A (mod m)
-    Matrix<Integer> multiplication_cut(const Matrix<Integer>& A, const size_t& c) const; // returns 
-    // this*(first c columns of A)
     bool equal(const Matrix& A) const;             // returns this==A
     bool equal(const Matrix& A, long m) const;     // returns this==A (mod m)
     Matrix transpose() const;                     // returns the transpose of this
@@ -407,7 +407,9 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
 //for simplicial subcones
 
     // computes support hyperplanes and volume
-    void simplex_data(const vector<key_t>& key, Matrix<Integer>& Supp, Integer& vol, bool compute_vol) const; 
+    void simplex_data(const vector<key_t>& key, Matrix<Integer>& Supp, Integer& vol, bool compute_vol) const;
+    // finds subdivision points
+    vector<Integer> optimal_subdivision_point() const;
     
 // Sorting of rows
     
